#!perl patch.pl
# usage: perl patch.pl original_file > outfile
# copyright (C) 1996-2000 Jun TAKASHIMA (tjun@aa.tufs.ac.jp)
# version 1.0 2000/03/30
#############################################################
# This program is different from patch in UNIX
# This read from original_file.cor to output a patched original_file
# The .cor file and original_file must have the same form like:
# -------------------------------------------------------------------
# tantraloka.1.2b/. maatRmaanaprameyaaMzazuulaambujakRtaaspadaam // 2
# -------------------------------------------------------------------
# title name with verse number and the verse itself separated by ``/.''
# The .cor file may have the form with initial `\' like:
# -------------------------------------------------------------------
# \tantraloka.1.2b/. maatRmaanaprameyaaMzazuulaambujakRtaaspadaam // 2
# -------------------------------------------------------------------
# if you want to put `\' mark manually, to make .cor file.
# Normally, it supposes the use of vdiff.pl to create the .cor file
# from the output of vdiff.pl like:
# -------------------------------------------------------------------
# new = tantraloka.32.40b/. eSa kramastato@nyo@pi vyutkramaH khecarii paraa // 40
# orig= tantraloka.32.40b/. eSa kramastato @nyo @pi vyutkramaH khecarii paraa // 40
# -------------------------------------------------------------------
# you leave only the desirable part of new and eliminate ``new = ''
# and then use that file as .cor file to introduce the changes into
# your file.
# ----------sample -------------------
# --sample.cor (sample patch file)-------------------------------
# \svacch.1.2b/. stuuyamaanaM mahezaanaM gaNamaatRniSevitam // 2
# svacch.1.4b/. yattvayaa kathitaM mahyaM svacchandaM paramezvara // 4
# --sample.num (sample original file)----------------------------
# svacch.1.1a/. kailaasazikharaasiinaM bhairavaM vigataamayam /
# svacch.1.1b/. caNDanandimahaakaalagaNezavRSabhRGgibhiH // 1
# svacch.1.2a/. kumaarendrayamaadityabrahmaviSNupuraHsaraiH /
# svacch.1.2b/. stuuyamaanaM maheza naM gaNamaatRniSevitam // 2
# svacch.1.3a/. sRSTisaMhaarakarttaaraM vilayasthitikaarakam /
# svacch.1.3b/. anugrahakaraM devaM praNataartivinaazanam // 3
# svacch.1.4a/. muditaM bhairavaM dRSTvaa devii vacanamabraviit /
# zriidevyuvaaca
# svacch.1.4b/. yattvayaa kthitaM mahyaM svacchandaM paramezvara // 4
# svacch.1.5a/. zatakoTipravistiirNaM bhedaanantyavisarpitam /
# ----------------------------------------------------------------
# the above original_file's two lines' contents will be replaced 
# by the corresponding two lines pf .cor file.
# ----------------------------------------------------------------
# A file with extension .p_c will be created to report what replacement
# was made.
# In the .cor file, lines without ``/.'' will be ignored.
################################################################
# ---------below explanation in Japanese --------------
# patch.pl - tB[h̃R[ȟ㔼u
# ̓t@Č`:   /. 
# ܂Aꎍߔԍ̖{݂̂̒û߂ɎgpB
# ̐lɍZĂĂeLXgf[^x[X̃eiX
# ̂߂̃c[łB
# usage: jperl patch.pl infile > outfile
# patch file ƂẮAinfile̊gq .corƊ̂I
# ǂݍނÃt@C݂ȂƃG[bZ[W\
# IB
# patch file ̂Ȃ /. ܂܂Ȃs͖B
# patch file ̍sɂ \ 邱Ƃ\肵Ă邪ȂƂǂB
# ͎̂悤sed scriptpatch file 𐶐邱Ƃ\肵Ăł
# -----c_ext.sed------
# /^\\/p
# d
# --------------------
# 
# --sample.cor (sample patch file)-------------------------------
# \svacch.1.2b/. stuuyamaanaM mahezaanaM gaNamaatRniSevitam // 2
# \svacch.1.4b/. yattvayaa kathitaM mahyaM svacchandaM paramezvara // 4
# --sample.num (sample original file)----------------------------
# svacch.1.1a/. kailaasazikharaasiinaM bhairavaM vigataamayam /
# svacch.1.1b/. caNDanandimahaakaalagaNezavRSabhRGgibhiH // 1
# svacch.1.2a/. kumaarendrayamaadityabrahmaviSNupuraHsaraiH /
# svacch.1.2b/. stuuyamaanaM maheza naM gaNamaatRniSevitam // 2
# svacch.1.3a/. sRSTisaMhaarakarttaaraM vilayasthitikaarakam /
# svacch.1.3b/. anugrahakaraM devaM praNataartivinaazanam // 3
# svacch.1.4a/. muditaM bhairavaM dRSTvaa devii vacanamabraviit /
# zriidevyuvaaca
# svacch.1.4b/. yattvayaa kthitaM mahyaM svacchandaM paramezvara // 4
# svacch.1.5a/. zatakoTipravistiirNaM bhedaanantyavisarpitam /
# ----------------------------------------------------------------
# Loriginal file 2s patch file ̑ΉsŒu
# ----------------------------------------------------------------
# ǂ̂悤Ȓusꂽ́Agq .p_c ̃t@Cɏo͂
# patchƂ͓삪Ⴄ̂Œ
#-----------------------------------------------------------------
{
	 $orig_file = $ARGV[0];
	($name,$ext)=split(/[.]/,$ARGV[0]);
	 $patchfile = $name . ".cor";
	 $checkfile = $name . ".p_c";
	 if (!open(patch,"<$patchfile")) { 
		die "cannot access patchfile \`$patchfile\'\n"; 
	 }
	 open(orig_file,"<$orig_file");
	 open(check_file,">$checkfile");
	 $tm_start = time;
}
pline: while (<patch>) {
	unless (/\/\./){next pline;}
	($pnum,$pcontent) = split(/\/\./, $_ ,2);
		$pnum =~ s/^\\//;
		line: while (<orig_file>) {
			$n_rec++;
			if (!($n_rec % 1000)) { 
				$tm_used= time - $tm_start;
				print STDERR "$n_rec lines, $c_count lines corrected, $tm_used sec...\n";
			}
			($num,$content) = split(/\/\./, $_ ,2);
			if ($num eq $pnum){
				print $pnum, "/.", $pcontent;
				print check_file "orig= ", $num, "/.", $content;
				print check_file "new = ", $pnum, "/.", $pcontent, "\n";
				$c_count++;
				next pline;
			}
			else {
				print;
#				next line;
			}
		}
}
{
	close(patch);
	print check_file "\n total number of lines corrected is ", $c_count, "\n";
	close(check_file);
	$tm_used= time - $tm_start;
	print STDERR "file correction finished,\n $n_rec lines, $c_count lines corrected, $tm_used sec...\n";
	print STDERR "now outputting the rest of original file...\n";
	while (<orig_file>) {
		print;
		$n_rec++;
		if (!($n_rec % 1000)) { 
			$tm_used= time - $tm_start;
			print STDERR "$n_rec lines, $tm_used sec...\n";
		}
	}
	$tm_used= time - $tm_start;
	print STDERR "$n_rec lines, $tm_used sec, output finished.\n";
	close(orig_file);
}
